function [perm] = permutationAlignmentDyadic(A, Y1, Y2, samplerate, print)
% PERMUTATIONALIGNMENTDYADIC Dyadic Sorting Permutation Alignment for BSS
%   This function performs permutation alignment for frequency-domain Blind
%   Source Separation of convolutive mixtures using the dyadic sorting
%   approach.

if ~exist('print', 'var') || isempty(print)
    print = 0; % Default: no printing
end

if print, disp('Permutation Alignment (dyadic): Start'); end

numFreqbins = length(Y1(:,1));
numTimeslots = length(Y1(1,:));

perm = repmat(eye(2), 1, 1, numFreqbins);
swap = [0 1; 1 0]; % permutation matrix for 2x2 case


%% Power Ratio Measure (Step 3)
if print, disp('  Calculate power ratio measures'); end
a1 = squeeze(A(:,1,:));
a2 = squeeze(A(:,2,:));
v1 = zeros(size(Y1));
v2 = zeros(size(Y2));
for freqbin = 1 : numFreqbins
    for timeslot = 1 : numTimeslots
        v1(freqbin,timeslot) = norm(a1(:,freqbin)*Y1(freqbin,timeslot))^2;
        v2(freqbin,timeslot) = norm(a2(:,freqbin)*Y2(freqbin,timeslot))^2;
        v_denominator = v1(freqbin,timeslot) + v2(freqbin,timeslot);
        v1(freqbin,timeslot) = v1(freqbin,timeslot) / v_denominator;
        v2(freqbin,timeslot) = v2(freqbin,timeslot) / v_denominator;
    end
end


%% Calculate number of frequency bins (Step 4)
treeLayers = floor(log2(numFreqbins));
numTreeFreqbins = 2^treeLayers;
if print, fprintf('  %d Layers, %d bins in tree\n', treeLayers, numTreeFreqbins); end


%% Divide Frequency Band (Step 5)
if print, disp('  Initialize groups'); end
indecies = cell(1,numTreeFreqbins);
powerratio1 = cell(1,numTreeFreqbins);
powerratio2 = cell(1,numTreeFreqbins);
for group = 1 : numTreeFreqbins
    indecies{group} = group;
    powerratio1{group} = v1(group,:);
    powerratio2{group} = v2(group,:);
end


%% Correct Permutation bin by bin and Merge Groups (Step 6 and 7)
if print, disp('  Merge groups'); end
groups = numTreeFreqbins;
while groups > 1
    
    for group = groups : -2 : 2
        
        if normCorrCoeff(powerratio1{group}, powerratio1{group-1}) < 0
            % swap
            for freqbin = indecies{group}
                perm(:,:,freqbin) = swap*perm(:,:,freqbin);
            end
            prtemp = powerratio1{group};
            powerratio1{group} = powerratio2{group};
            powerratio2{group} = prtemp;
        end
        
        indecies{group-1} = [indecies{group-1} indecies{group}];
        indecies(group) = [];
        
        powerratio1{group-1} = (powerratio1{group-1}+powerratio1{group})/2;
        powerratio1(group) = [];
        
        powerratio2{group-1} = (powerratio2{group-1}+powerratio2{group})/2;
        powerratio2(group) = [];
    end
    
    groups = groups / 2;
end

if print, disp('Permutation Alignment (dyadic): Finish'); end

end % function
